( cd usr/bin ; rm -rf wx-config )
( cd usr/bin ; ln -sf wxgtk-2.4-config wx-config )
( cd usr/lib ; rm -rf libwx_gtk-2.4.so.0 )
( cd usr/lib ; ln -sf libwx_gtk-2.4.so.0.1.1 libwx_gtk-2.4.so.0 )
( cd usr/lib ; rm -rf libwx_gtk-2.4.so )
( cd usr/lib ; ln -sf libwx_gtk-2.4.so.0.1.1 libwx_gtk-2.4.so )
